@@ -99,7 +99,7 @@ sub find_candidate_packages {
 sub get_installed_arch {
     my ($db, $n) = @_;
     my $arch;
-    $db->traverse_tag_find('name', $n, sub { $arch = $_[0]->arch });
+    $db->traverse_tag_find('name', $n, sub { $arch = $_[0]->arch; 1 });
     $arch;
 }
 
@@ -162,7 +162,7 @@ sub _is_selected_or_installed {
     my ($urpm, $db, $name) = @_;
 
     (grep { $_->flag_available } $urpm->packages_providing($name)) > 0 ||
-      $db->traverse_tag_find('name', $name, sub {}) > 0;
+      $db->traverse_tag('name', [ $name ], undef) > 0;
 }
 
 #- finds $pkg "provides" that matches $provide_name, and returns the version provided
@@ -217,12 +217,11 @@ sub find_required_package {
 	    $may_add_to_packages->($pkg);
 	} elsif (my $name = property2name($_)) {
 	    my $property = $_;
-	    foreach (sort { $a <=> $b } keys %{$urpm->{provides}{$name} || {}}) {
-		my $pkg = $urpm->{depslist}[$_];
+	    foreach my $pkg (packages_providing($urpm, $name)) {
 		$pkg->is_arch_compat or next;
 		$pkg->flag_skip || $state->{rejected}{$pkg->fullname} and next;
 		#- check if at least one provide of the package overlaps the property
-		if (!$urpm->{provides}{$name}{$_} || $pkg->provides_overlap($property)) {
+		if (!$urpm->{provides}{$name}{$pkg->id} || $pkg->provides_overlap($property)) {
 		    #- determine if this package is better than a possibly previously chosen package.
 		    $pkg->flag_selected || exists $state->{selected}{$pkg->id} and return [$pkg];
 		    !$strict_arch || strict_arch_check_installed($db, $pkg) or next;
@@ -967,6 +966,7 @@ sub resolve_requested__no_suggests_ {
     #- then examine provides that will be removed (which need to be satisfied by another
     #- package present or by a new package to upgrade), then requires not satisfied and
     #- finally conflicts that will force a new upgrade or a remove.
+    my $count = 1;
     do {
 	while (my $dep = shift @properties) {
 	    #- we need to avoid selecting packages if the source has been disabled.
@@ -1054,7 +1054,13 @@ sub resolve_requested__no_suggests_ {
 	} elsif (my $dep = shift @choices) {
 	    push @properties, $dep;
 	}
-    } while @diff_provides || @properties || @choices;
+
+	# safety:
+	if ($count++ > 50000) {
+	    die("detecting looping forever while trying to resolve dependancies.\n"
+		. "Aborting... Try again with '-vv --debug' options");
+	}
+    } while (@diff_provides || @properties || @choices);
 
     #- return what has been selected by this call (not all selected hash which may be not empty
     #- previously. avoid returning rejected packages which weren't selectable.
@@ -2004,7 +2010,7 @@ sub build_transaction_set {
 
     if ($state->{orphans_to_remove}) {
 	my @l = map { scalar $_->fullname } @{$state->{orphans_to_remove}};
-	push @{$state->{transaction}}, { remove  => \@l };
+	push @{$state->{transaction}}, { remove  => \@l } if @l;
     }
 
     $state->{transaction};
